//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <stddef.h>
#include "typedefs.h"
#include "msp430f2274.h"
#include "i2c.h"
#include "stdbool.h"
//-----------------------------------------------------------------------------
// Defines
//-----------------------------------------------------------------------------
#define READ    1
#define WRITE   0
#define IO3W_CTLWORD_WRITE  (0x00)
#define IO3W_CTLWORD_READ   (0x20)
#define IO3W_ADDR           (0xA0)

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Function prototypes
//-----------------------------------------------------------------------------

void _nop_(void);

//-----------------------------------------------------------------------------
// Externals
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// This routine writes the address prior to writing any data to the part
//
// Inputs:
//       addr:      The address to write
//       operation: Specifies read or write operation
//-----------------------------------------------------------------------------
io3w_rawAddr(unsigned char addr, unsigned char operation)
{
    unsigned char i;
    int controlWord = 0x0000;

    //SDIO_OUT |= SDIO_HEX;  // Configure SDIO as push-pull

    // Make room for READ/WRITE by shifting most sigificant 3 bits up.
    controlWord |= ((int)(addr & 0xE0) << 1) | (addr & 0x1F);  
	
	// Add the READ/WRITE bit to the control word
    controlWord |= (operation == READ) ? IO3W_CTLWORD_READ : IO3W_CTLWORD_WRITE;
	controlWord |= (addr & 0x1F);

    SCL_low();	//SCLK = 0;
    SEN_low();	//SENB = 0; Start bertragung
    
    for (i=9; i>0; i--)
    {
        SCL_low();	//SCLK = 0;
        SDA_OUT((controlWord >> i-1) & 0x01); //SDIO = ((controlWord >> i) & 0x01);
        SCL_high();	//SCLK = 1;SCLK = 1;
    }
}

//-----------------------------------------------------------------------------
// This routine writes one byte of data onto the 3 wire interface
//
// Inputs:
//       data_out: The data to write
//-----------------------------------------------------------------------------
void io3w_rawDataOut(unsigned char data_out)
{
    unsigned char i;
    for (i=8; i>0; i--)
    {
        SCL_low();	//SCLK = 0;
        SDA_OUT((data_out >> i-1) & 0x01); //SDIO = ((data_out >> i) & 0x01);
        SCL_high();	//SCLK = 1;
    }

    // Wait for the part to write the data
    wait_us(10);
}

//-----------------------------------------------------------------------------
// This routine reads one byte of data from the 3 wire interface
//
// Inputs:
//       data_in: The variable containing the data to read
//-----------------------------------------------------------------------------
void io3w_rawDataIn(unsigned char *data_in)
{
    unsigned char i;

    // Wait for control ready / time
	wait_us(10);

    P3REN &= 0xFB;
    P2REN |= 0x04;  //GPIO PullUp
	SDA_read();

    for (i=0; i<8; i++)
    {
        SCL_low(); //SCLK = 0;
        SCL_high(); //SCLK = 1;
        *data_in = ((*data_in << 1) | SDA_IN());
    }
}

//-----------------------------------------------------------------------------
// This routine stops the 3 wire command
//-----------------------------------------------------------------------------
void io3w_rawStop(void) 
{
    // end the transaction
    SCL_low(); //SCLK = 0;
	SEN_high();		//SENB = 1; Ende bertragung
    SCL_high();	//SCLK = 1;
	SCL_low(); //SCLK = 0;

    P3REN |= 0x04; //SDIO_OUT |= SDIO_HEX;  // Configure SDIO as push-pull
    SDA_OUT(1); //SDIO = 1;
}

//-----------------------------------------------------------------------------
// This routine transfers 2 bytes of data between the host and Si47xx.
//
// Inputs:
//		operation:  Specifies read or write operation
//		addr:		Address for register read or write
//		data_value:  The value of the data to write
//
// Pin drive status upon exit:
//      SDIO = high (push-pull output)
//      SCLK = low
//      SENB = high
//-----------------------------------------------------------------------------
void io3w_operation(unsigned char operation, unsigned char addr, int *data_value)
{
	io3w_rawAddr(addr, operation);  // Write the address to read or write

    // read or write data
    if (operation == WRITE)
    {
        io3w_rawDataOut((unsigned char)(*data_value >> 8));  // Write the high byte
        io3w_rawDataOut((unsigned char)(*data_value >> 0));  // Write the low byte
    }
    else  // READ
    {
        io3w_rawDataIn(((unsigned char*)data_value) + 1);
        io3w_rawDataIn(((unsigned char*)data_value) + 0);
    }

	io3w_rawStop();  // Stop the operation
}

//-----------------------------------------------------------------------------
// This routine reads an array of bytes from the si47xx part
//
// Inputs:
//		number_bytes: Number of bytes to read
//		data_in:	Destination array for data read
//
//-----------------------------------------------------------------------------
void io3w_read(unsigned char number_bytes, unsigned char *data_in)
{
	int tmp;
    unsigned char i;

	// Loop and read back the 16 bit data from the part and convert to 8 bit data.
    for (i=0; i < (number_bytes+1)/2; i++)
    {
        io3w_operation(READ, IO3W_ADDR | (i+8), &tmp);
		data_in[(i*2)+0] = (unsigned char)(tmp >> 8);
        if (i < number_bytes)
        {
            data_in[(i*2)+1] = (unsigned char)(tmp >> 0);
        }
    }
}

//-----------------------------------------------------------------------------
// This routine writes an array of bytes to the Si47xx.
//
// Inputs:
//		number_bytes: Number of bytes to write
//		data_out:	  Source array for data to be written
//
//-----------------------------------------------------------------------------
void io3w_write(u8 number_bytes, u8 *data_out)
{
	int regData[8] = {0,0,0,0,0,0,0,0};
	int tmp;
	unsigned char i;

	// Take up to 16 elements in the input array and convert those
    // values to a 8 element 16 bit array
    for (i=0; i<number_bytes; i++)
    {
        tmp = (int)(data_out[i]);
        regData[i/2] |= tmp << (8 * (1 - (i%2)));
    }

    // Now write the elements via the 3w interface
    for (i=1; i<(number_bytes+1)/2; i++)
    {
        io3w_operation(WRITE, IO3W_ADDR | i, &(regData[i]));
    }
	io3w_operation(WRITE, IO3W_ADDR | 0, &(regData[0]));
}
